package runtime;

import java.util.Map;
import java.util.HashMap;


public class BooleanLockManager
{
	private static Map<String, BooleanLock> varname2lock;
	
	static
	{
		varname2lock = new HashMap<String, BooleanLock>();
	}
	
	
	public static void lock(String varName)
	{
		BooleanLock lockObj = getLockObject(varName);
		
		lockObj.status = true;
	}
	
	public static boolean isLocked(String varName)
	{
		boolean status = false;
		
		BooleanLock lockObj = getLockObject(varName);

		status = lockObj.status;
		
		return status;
	}
	
	public static void waitForLock(String varName)
	{
		synchronized (varname2lock)
		{
			try
			{
				varname2lock.wait();
			}
			catch (InterruptedException ex) {}
		}
	}
		
	public static void unlock(String varName)
	{
		BooleanLock lockObj = getLockObject(varName);
		
		lockObj.status = false;
		
		synchronized (varname2lock)
		{
			varname2lock.notify();
		}
	}
	
	public static void wait(String varName)
	{
		BooleanLock lockObj = null;
		
		lockObj = getLockObject(varName);
		
		synchronized (lockObj)
		{
			try
			{
				lockObj.wait();
			}
			catch (InterruptedException ex) {}
		}
	}
	
	public static void notify(String varName)
	{
		BooleanLock lockObj = null;
		
		lockObj = getLockObject(varName);
		
		synchronized (lockObj)
		{
			lockObj.notify();
		}
	}

	
	private static BooleanLock getLockObject(String varName)
	{
		BooleanLock lockObj	= varname2lock.get(varName);
		
		if (lockObj == null) 
		{
			lockObj = new BooleanLock(varName);
			
			varname2lock.put(varName, lockObj);
		}
		
		return lockObj;
	}
	
	
	static class BooleanLock
	{
		public String varName;
		public boolean status;
		
		public BooleanLock(String varName)
		{
			this.varName = varName;
			this.status = false;
		}
	}
}
